@@ -42,7 +42,9 @@ AWS1G_MODEL = "AWS EC2"
 AZURE2G_MODEL = "Microsoft Azure Cloud Provider 2G"
 AWS2G_MODEL = "Amazon AWS Cloud Provider 2G"
 
-CLP_PROVIDER_MODELS = [AWS1G_MODEL, AWS2G_MODEL, AZURE1G_MODEL, AZURE2G_MODEL]
+GCP2G_MODEL = "Google Cloud Provider"
+
+CLP_PROVIDER_MODELS = [AWS1G_MODEL, AWS2G_MODEL, AZURE1G_MODEL, AZURE2G_MODEL, GCP2G_MODEL]
 
 # Misc
 DIRTY_CHARS = r'''
