@@ -65,3 +65,24 @@ class AzureCloudProviderEnvVarHandler(BaseCloudProviderEnvVarHandler):
                 os.environ["ARM_CLIENT_ID"] = attr.Value
             if self.does_attribute_match(self._clp_res_model, attr, "Azure Application Key"):
                 os.environ["ARM_CLIENT_SECRET"] = self._shell_helper.api.DecryptPassword(attr.Value).Value
+
+
+class GCPCloudProviderEnvVarHandler(BaseCloudProviderEnvVarHandler):
+    def __init__(self, clp_res_model, clp_resource_attributes, shell_helper):
+        BaseCloudProviderEnvVarHandler.__init__(self)
+        self._clp_res_model = clp_res_model
+        self._clp_resource_attributes = clp_resource_attributes
+        self._shell_helper = shell_helper
+
+    def set_env_vars_based_on_clp(self):
+        project_flag = False
+        cred_flag = False
+        for attr in self._clp_resource_attributes:
+            if self.does_attribute_match(self._clp_res_model, attr, "Google Cloud Provider.Credentials Json Path"):
+                os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = attr.Value
+                cred_flag = True
+            if self.does_attribute_match(self._clp_res_model, attr, "Google Cloud Provider.project"):
+                os.environ["GOOGLE_PROJECT"] = attr.Value
+                project_flag = True
+        if not cred_flag and not project_flag:
+            self._shell_helper.sandbox_messages.write_message("Project ID was not found on GCP Cloud Provider")
